/*  gtklepin - a gtk+ notebook
 *
 *  Copyright (C) 20002  Luc Dufresne <luc@ordiluc.net>
 *                       26, rue des Comices
 *                       59650 Villeneuve d'Ascq
 *                       FRANCE
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  See the file COPYING
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#ifdef _WIN32
#include <direct.h>
#include "win32.h"
#else
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <gtk/gtk.h>

#include "interface.h"
#include "support.h"
#include "options.h"
#include "misc.h"
#include "boulot.h"

GtkWidget *options_window;

static void hide_radios (gint i) {
	GtkWidget *label, *radiobutton_both, *radiobutton_pictures, *radiobutton_text;

	radiobutton_both=lookup_widget (options_window, "radiobutton_both");
	radiobutton_pictures=lookup_widget (options_window, "radiobutton_pictures");
	radiobutton_text=lookup_widget (options_window, "radiobutton_text");
	label=lookup_widget (options_window, "label_toolbar");
	
	gtk_widget_set_sensitive(radiobutton_both,i);
	gtk_widget_set_sensitive(radiobutton_pictures,i);
	gtk_widget_set_sensitive(radiobutton_text,i);
	gtk_widget_set_sensitive(label,i);
}

static void hide_toolbar (gint i) {
	GtkWidget *toolbar;
	toolbar=lookup_widget (options.main_window, "toolbar");
	if(i) gtk_widget_show(toolbar); else gtk_widget_hide(toolbar);
}

void toolbar_style (gint i) {
        GtkToolbar *toolbar;
	toolbar=(GtkToolbar *)lookup_widget (options.main_window, "toolbar");
	switch (i) {
		case 0:
			gtk_toolbar_set_style(toolbar,GTK_TOOLBAR_BOTH);
			break;
		case 1:
			gtk_toolbar_set_style(toolbar,GTK_TOOLBAR_ICONS);
			break;
		case 2:
			gtk_toolbar_set_style(toolbar,GTK_TOOLBAR_TEXT);
			break;
	}
}

void tabs_pos (gint i) {
        GtkNotebook *notebook;
	notebook=(GtkNotebook *)lookup_widget (options.main_window, "notebook");
	switch (i) {
		case 0:
			gtk_notebook_set_tab_pos(notebook,GTK_POS_LEFT);
			break;
		case 1:
			gtk_notebook_set_tab_pos(notebook,GTK_POS_RIGHT);
			break;
		case 2:
			gtk_notebook_set_tab_pos(notebook,GTK_POS_TOP);
			break;
		case 3:
			gtk_notebook_set_tab_pos(notebook,GTK_POS_BOTTOM);
			break;
			
	}
}

static void do_line_wrap (gpointer data, gpointer i) {
	gint *wrap;
	struct datastruct *nb_data;
	
	wrap = (gint *) (i);
	nb_data = (struct datastruct *) data;
	gtk_text_set_line_wrap ((GtkText *) nb_data->text, *wrap);
}

void line_wrap (gboolean i, GList *nb_data) {
	g_list_foreach (nb_data, do_line_wrap, &i);
}

void open_options_window (gint page) {
	options_window = create_options_window ();
	if (!options.ShowToolbar) hide_radios (0);
	else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (lookup_widget (options_window, "checkbutton_toolbar")), TRUE);

	switch (options.ToolBarType) {
		case 0:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_both")), TRUE);
			break;
		case 1:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_pictures")), TRUE);
			break;
		case 2:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_text")), TRUE);
			break;
	}

	switch (options.TabPos) {
		case 0:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_left")), TRUE);
			break;
		case 1:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_right")), TRUE);
			break;
		case 2:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_top")), TRUE);
			break;
		case 3:
			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "radiobutton_bottom")), TRUE);
			break;
	}
	
	if (options.OpenLast) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "checkbutton_openlast")), TRUE);
	if (options.LineWarp) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "checkbutton_linewarp")), TRUE);	
	if (options.Backup) gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (lookup_widget (options_window, "checkbutton_backup")), TRUE);	
	
	gtk_notebook_set_page (GTK_NOTEBOOK(lookup_widget (options_window, "notebook_options")), page);
	gtk_widget_grab_default (lookup_widget (options_window, "button_ok"));
	gtk_widget_show (options_window);
}

void show_toolbar_toggle (GtkToggleButton *button) {
	if (GTK_TOGGLE_BUTTON (button)->active)	{
		hide_radios (1);
		hide_toolbar (1);
        }
	else {
		hide_radios (0);
		hide_toolbar (0);
        }
}

void save_options() {
	GString *buf;
	FILE *fichier;
	
#ifdef _WIN32
	buf=g_string_new("options");
#else
	buf=g_string_new("");
	g_string_sprintf(buf,"%s/."PACKAGE"",getenv("HOME"));
	mkdir(buf->str,S_IRWXU);
	g_string_sprintf(buf,"%s/."PACKAGE"/options",getenv("HOME"));
#endif
	if(NULL!=(fichier=fopen(buf->str,"wt"))) {
		fprintf(fichier,"# Automatically generated file.\n# Do not edit (or make a backup copy before doing so)\n");
		fprintf(fichier,"toolbar:%d\n",options.ShowToolbar);
		fprintf(fichier,"toolbartype:%d\n",options.ToolBarType);
		fprintf(fichier,"tabpos:%d\n",options.TabPos);
                fprintf(fichier,"openlast:%d\n",options.OpenLast);
		fprintf(fichier,"backup:%d\n",options.Backup);
		fprintf(fichier,"linewarp:%d\n",options.LineWarp);
		if (options.LastOpenedFile->str != NULL) fprintf(fichier,"lastopenedfile:%s\n", options.LastOpenedFile->str);
		else fprintf(fichier,"lastopenedfile:\n");
		fclose(fichier);
	
	}
	g_string_free(buf,TRUE);
}

void options_ok() {
	GtkWidget *radiobutton_both, *radiobutton_pictures, *radiobutton_text, *radiobutton_left, *radiobutton_right, *radiobutton_top, *radiobutton_bottom, *show_toggle, *open_last, *line_warp, *backup;

	show_toggle = lookup_widget (options_window, "checkbutton_toolbar");
	radiobutton_both = lookup_widget (options_window, "radiobutton_both");
	radiobutton_pictures = lookup_widget (options_window, "radiobutton_pictures");
	radiobutton_text = lookup_widget (options_window, "radiobutton_text");
	radiobutton_left = lookup_widget (options_window, "radiobutton_left");
	radiobutton_right = lookup_widget (options_window, "radiobutton_right");
	radiobutton_top = lookup_widget (options_window, "radiobutton_top");
	radiobutton_bottom = lookup_widget (options_window, "radiobutton_bottom");	
	open_last = lookup_widget (options_window, "checkbutton_openlast");
	line_warp = lookup_widget (options_window, "checkbutton_linewarp");
	backup = lookup_widget (options_window, "checkbutton_backup");

	options.ShowToolbar = GTK_TOGGLE_BUTTON (show_toggle)->active ? 1 : 0;
	if (GTK_TOGGLE_BUTTON (radiobutton_both)->active) options.ToolBarType=0;
	if (GTK_TOGGLE_BUTTON (radiobutton_pictures)->active) options.ToolBarType=1;
	if (GTK_TOGGLE_BUTTON (radiobutton_text)->active) options.ToolBarType=2;
	if (GTK_TOGGLE_BUTTON (radiobutton_left)->active) options.TabPos=0;
	if (GTK_TOGGLE_BUTTON (radiobutton_right)->active) options.TabPos=1;
	if (GTK_TOGGLE_BUTTON (radiobutton_top)->active) options.TabPos=2;
	if (GTK_TOGGLE_BUTTON (radiobutton_bottom)->active) options.TabPos=3;
	options.OpenLast = GTK_TOGGLE_BUTTON (open_last)->active ? 1 : 0;
	options.LineWarp = GTK_TOGGLE_BUTTON (line_warp)->active ? 1 : 0;
	options.Backup = GTK_TOGGLE_BUTTON (backup)->active ? 1 : 0;

	save_options();
	gtk_widget_destroy(options_window);	
}

void options_cancel() {
	gtk_widget_destroy(options_window);
	hide_toolbar(options.ShowToolbar);
	toolbar_style(options.ToolBarType);
	tabs_pos(options.TabPos);
	line_wrap_callback(options.LineWarp);
}

void read_options(GtkWidget *window, GList *nb_data) {
	FILE *fichier;
	GString *buf,*buf2;
	gchar *c;
	gchar *tmp;
	
	options.ShowToolbar=1;
	options.ToolBarType=0;
	options.OpenLast=1;
	options.LineWarp=1;
	options.Backup=1;
	options.TabPos=1;
	options.main_window=window;
	options.LastOpenedFile=g_string_new("");

#ifdef _WIN32
	buf=g_string_new("options");
#else
	buf=g_string_new("");
	g_string_sprintf(buf,"%s/."PACKAGE"/options",getenv("HOME"));
#endif
	if(NULL==(fichier=fopen(buf->str,"rt"))) return;
	while(!feof(fichier)) {
	        g_string_free(buf,TRUE);
		buf=getline(fichier);
                if(feof(fichier)) break;
		if(buf->str[0]!='#') {
			buf2=g_string_new("");
			c=buf->str;
			while(*c!=':' && *c!='\n') {
				g_string_append_c(buf2,*c);
				c++;
			}
			c++;
/*			printf("%s   %s\n",buf2->str,c);
*/			if (0==strcmp(buf2->str,"toolbar")) options.ShowToolbar = atoi(c);
			if (0==strcmp(buf2->str,"toolbartype")) options.ToolBarType = atoi(c);
			if (0==strcmp(buf2->str,"openlast")) options.OpenLast = atoi(c);
			if (0==strcmp(buf2->str,"linewarp")) options.LineWarp = atoi(c);
			if (0==strcmp(buf2->str,"backup")) options.Backup = atoi(c);
			if (0==strcmp(buf2->str,"tabpos")) options.TabPos = atoi(c);
			if (0==strcmp(buf2->str,"lastopenedfile")) {
				tmp = strtok (c,"\n");
				if (tmp != NULL) g_string_sprintf(options.LastOpenedFile, "%s", strtok(c,"\n"));
			}
			g_string_free(buf2,TRUE);
		}
	}
	fclose(fichier);
	g_string_free(buf,TRUE);
/*	printf("%d %d %d \n",options.ShowToolbar,options.ToolBarType,options.OpenLast);
*/	hide_toolbar(options.ShowToolbar);
	toolbar_style(options.ToolBarType);
	tabs_pos(options.TabPos);
	line_wrap(options.LineWarp, nb_data);
}
